  function [V,E,F] = read_mesh( filename )
% function [V,E,F] = read_mesh( filename )
%
% Loads a mesh in Wavefront OBJ format.
%
% INPUT:
%    filename - path to mesh file
%
% OUTPUT:
%    V - dense 2x|V| matrix of vertex positions
%    E - dense |E|x2 matrix of edges as 1-based indices into vertex list; smaller index comes first
%    F - dense |F|x3 matrix of faces as 1-based indices into edge list; indices are negated depending on orientation relative to face
%

   fp = fopen( filename, 'r' );
   if( fp == -1 )
      disp( sprintf( 'Error: could not read mesh file "%s"\n', filename ));
      return;
   end

   v = zeros( 2, 0 );
   f = zeros( 0, 3 );
   while( ~feof( fp ))
       line = fscanf( fp, '%c %f %f %f' );
       label = line( 1 );
       values = line( 2:4 );

       if( label == 'v' ) % vertex
          v = [ v, values(1:2) ];
       end

       if( label == 'f' ) % face
          f = [ f; values' ];
       end
   end
   fclose( fp );

   V = v;
   E = zeros( 0, 2 );
   F = zeros( size(f,1), 3 );

   edgeIndex = sparse( size(v,2), size(v,2) );
   nEdges = 0;

   for i = 1:size( f, 1 )
      for j = 0:2
         v1 = f( i, 1+j );
         v2 = f( i, 1+mod(j+1,3) );
         if( v2 < v1 )
            tmp = v1;
            v1 = v2;
            v2 = tmp;
            sgn = -1;
         else
            sgn = 1;
         end

         if( edgeIndex( v1, v2 ) == 0 )
            nEdges = nEdges+1;
            edgeIndex( v1, v2 ) = nEdges;
            E = [ E; [ v1, v2 ]];
         end

         F( i, 1+j ) = sgn*edgeIndex( v1, v2 );
      end
   end
end

